Exception: ShellTest::ShellMethods::Agent::ReadError
- Inherits:
-
RuntimeError
- Object
- RuntimeError
- ShellTest::ShellMethods::Agent::ReadError
- Defined in:
- lib/shell_test/shell_methods/agent.rb
Overview
Raised when Agent reads fail or timeout.
Instance Attribute Summary collapse
-
#buffer ⇒ Object
readonly
The buffer containing whatever had been read at the time of error.
Instance Method Summary collapse
-
#initialize(message, buffer) ⇒ ReadError
constructor
A new instance of ReadError.
Constructor Details
#initialize(message, buffer) ⇒ ReadError
Returns a new instance of ReadError.
126 127 128 129 |
# File 'lib/shell_test/shell_methods/agent.rb', line 126 def initialize(, buffer) @buffer = buffer super end |
Instance Attribute Details
#buffer ⇒ Object (readonly)
The buffer containing whatever had been read at the time of error.
124 125 126 |
# File 'lib/shell_test/shell_methods/agent.rb', line 124 def buffer @buffer end |